# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.exceptions import AirflowNotFoundException
from airflow.providers.google.marketing_platform.operators.analytics_admin import (
    GoogleAnalyticsAdminCreateDataStreamOperator,
    GoogleAnalyticsAdminCreatePropertyOperator,
    GoogleAnalyticsAdminDeleteDataStreamOperator,
    GoogleAnalyticsAdminDeletePropertyOperator,
    GoogleAnalyticsAdminGetGoogleAdsLinkOperator,
    GoogleAnalyticsAdminListAccountsOperator,
    GoogleAnalyticsAdminListGoogleAdsLinksOperator,
)

GCP_CONN_ID = "google_cloud_default"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]
TEST_GA_GOOGLE_ADS_PROPERTY_ID = "123456789"
TEST_GA_GOOGLE_ADS_LINK_ID = "987654321"
TEST_GA_GOOGLE_ADS_LINK_NAME = (
    f"properties/{TEST_GA_GOOGLE_ADS_PROPERTY_ID}/googleAdsLinks/{TEST_GA_GOOGLE_ADS_LINK_ID}"
)
TEST_PROPERTY_ID = "123456789"
TEST_PROPERTY_NAME = f"properties/{TEST_PROPERTY_ID}"
TEST_DATASTREAM_ID = "987654321"
TEST_DATASTREAM_NAME = f"properties/{TEST_PROPERTY_ID}/dataStreams/{TEST_DATASTREAM_ID}"
ANALYTICS_PATH = "airflow.providers.google.marketing_platform.operators.analytics_admin"


class TestGoogleAnalyticsAdminListAccountsOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    @mock.patch(f"{ANALYTICS_PATH}.Account.to_dict")
    def test_execute(self, account_to_dict_mock, hook_mock):
        list_accounts_returned = (mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        hook_mock.return_value.list_accounts.return_value = list_accounts_returned

        list_accounts_serialized = [mock.MagicMock(), mock.MagicMock(), mock.MagicMock()]
        account_to_dict_mock.side_effect = list_accounts_serialized

        mock_page_size, mock_page_token, mock_show_deleted, mock_retry, mock_timeout, mock_metadata = (
            mock.MagicMock() for _ in range(6)
        )

        retrieved_accounts_list = GoogleAnalyticsAdminListAccountsOperator(
            task_id="test_task",
            page_size=mock_page_size,
            page_token=mock_page_token,
            show_deleted=mock_show_deleted,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.list_accounts.assert_called_once_with(
            page_size=mock_page_size,
            page_token=mock_page_token,
            show_deleted=mock_show_deleted,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        account_to_dict_mock.assert_has_calls([mock.call(item) for item in list_accounts_returned])
        assert retrieved_accounts_list == list_accounts_serialized


class TestGoogleAnalyticsAdminCreatePropertyOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsPropertyLink")
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    @mock.patch(f"{ANALYTICS_PATH}.Property.to_dict")
    def test_execute(self, property_to_dict_mock, hook_mock, _):
        property_returned = mock.MagicMock()
        hook_mock.return_value.create_property.return_value = property_returned

        property_serialized = mock.MagicMock()
        property_to_dict_mock.return_value = property_serialized

        mock_property, mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(4))

        property_created = GoogleAnalyticsAdminCreatePropertyOperator(
            task_id="test_task",
            analytics_property=mock_property,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.create_property.assert_called_once_with(
            analytics_property=mock_property,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        property_to_dict_mock.assert_called_once_with(property_returned)
        assert property_created == property_serialized


class TestGoogleAnalyticsAdminDeletePropertyOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    @mock.patch(f"{ANALYTICS_PATH}.Property.to_dict")
    def test_execute(self, property_to_dict_mock, hook_mock):
        property_returned = mock.MagicMock()
        hook_mock.return_value.delete_property.return_value = property_returned

        property_serialized = mock.MagicMock()
        property_to_dict_mock.return_value = property_serialized

        mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(3))

        property_deleted = GoogleAnalyticsAdminDeletePropertyOperator(
            task_id="test_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            property_id=TEST_PROPERTY_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.delete_property.assert_called_once_with(
            property_id=TEST_PROPERTY_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        property_to_dict_mock.assert_called_once_with(property_returned)
        assert property_deleted == property_serialized


class TestGoogleAnalyticsAdminCreateDataStreamOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    @mock.patch(f"{ANALYTICS_PATH}.DataStream.to_dict")
    def test_execute(self, data_stream_to_dict_mock, hook_mock):
        data_stream_returned = mock.MagicMock()
        hook_mock.return_value.create_data_stream.return_value = data_stream_returned

        data_stream_serialized = mock.MagicMock()
        data_stream_to_dict_mock.return_value = data_stream_serialized

        mock_parent, mock_data_stream, mock_retry, mock_timeout, mock_metadata = (
            mock.MagicMock() for _ in range(5)
        )

        data_stream_created = GoogleAnalyticsAdminCreateDataStreamOperator(
            task_id="test_task",
            property_id=TEST_PROPERTY_ID,
            data_stream=mock_data_stream,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.create_data_stream.assert_called_once_with(
            property_id=TEST_PROPERTY_ID,
            data_stream=mock_data_stream,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        data_stream_to_dict_mock.assert_called_once_with(data_stream_returned)
        assert data_stream_created == data_stream_serialized


class TestGoogleAnalyticsAdminDeleteDataStreamOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    def test_execute(self, hook_mock):
        mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(3))

        GoogleAnalyticsAdminDeleteDataStreamOperator(
            task_id="test_task",
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            property_id=TEST_PROPERTY_ID,
            data_stream_id=TEST_DATASTREAM_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.delete_data_stream.assert_called_once_with(
            property_id=TEST_PROPERTY_ID,
            data_stream_id=TEST_DATASTREAM_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )


class TestGoogleAnalyticsAdminListGoogleAdsLinksOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAdsLink.to_dict")
    def test_execute(self, ads_link_to_dict_mock, hook_mock):
        list_ads_links_returned = (mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        hook_mock.return_value.list_google_ads_links.return_value = list_ads_links_returned

        list_ads_links_serialized = [mock.MagicMock(), mock.MagicMock(), mock.MagicMock()]
        ads_link_to_dict_mock.side_effect = list_ads_links_serialized

        mock_page_size, mock_page_token, mock_show_deleted, mock_retry, mock_timeout, mock_metadata = (
            mock.MagicMock() for _ in range(6)
        )

        retrieved_ads_links = GoogleAnalyticsAdminListGoogleAdsLinksOperator(
            task_id="test_task",
            property_id=TEST_PROPERTY_ID,
            page_size=mock_page_size,
            page_token=mock_page_token,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.list_google_ads_links.assert_called_once_with(
            property_id=TEST_PROPERTY_ID,
            page_size=mock_page_size,
            page_token=mock_page_token,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        ads_link_to_dict_mock.assert_has_calls([mock.call(item) for item in list_ads_links_returned])
        assert retrieved_ads_links == list_ads_links_serialized


class TestGoogleAnalyticsAdminGetGoogleAdsLinkOperator:
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    @mock.patch(f"{ANALYTICS_PATH}.GoogleAdsLink")
    def test_execute(self, mock_google_ads_link, hook_mock):
        mock_ad_link = mock.MagicMock()
        mock_ad_link.name = TEST_GA_GOOGLE_ADS_LINK_NAME
        list_ads_links = hook_mock.return_value.list_google_ads_links
        list_ads_links.return_value = [mock_ad_link]
        mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(3))

        GoogleAnalyticsAdminGetGoogleAdsLinkOperator(
            task_id="test_task",
            property_id=TEST_GA_GOOGLE_ADS_PROPERTY_ID,
            google_ads_link_id=TEST_GA_GOOGLE_ADS_LINK_ID,
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.list_google_ads_links.assert_called_once_with(
            property_id=TEST_PROPERTY_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        mock_google_ads_link.to_dict.assert_called_once_with(mock_ad_link)

    @mock.patch(f"{ANALYTICS_PATH}.GoogleAnalyticsAdminHook")
    def test_execute_not_found(self, hook_mock):
        list_ads_links = hook_mock.return_value.list_google_ads_links
        list_ads_links.return_value = []
        mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(3))

        with pytest.raises(AirflowNotFoundException):
            GoogleAnalyticsAdminGetGoogleAdsLinkOperator(
                task_id="test_task",
                gcp_conn_id=GCP_CONN_ID,
                impersonation_chain=IMPERSONATION_CHAIN,
                property_id=TEST_GA_GOOGLE_ADS_PROPERTY_ID,
                google_ads_link_id=TEST_GA_GOOGLE_ADS_LINK_ID,
                retry=mock_retry,
                timeout=mock_timeout,
                metadata=mock_metadata,
            ).execute(context=None)

        hook_mock.assert_called_once()
        hook_mock.return_value.list_google_ads_links.assert_called_once_with(
            property_id=TEST_PROPERTY_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import re
from collections.abc import Sequence
from copy import deepcopy
from typing import TYPE_CHECKING
from unittest import mock

import pytest
from google.api_core.retry import Retry
from google.cloud.datacatalog import CreateTagRequest, CreateTagTemplateRequest, Entry, Tag, TagTemplate
from google.protobuf.field_mask_pb2 import FieldMask

from airflow.exceptions import AirflowException, AirflowProviderDeprecationWarning
from airflow.providers.google.cloud.hooks.datacatalog import CloudDataCatalogHook

from unit.google.cloud.utils.base_gcp_mock import (
    mock_base_gcp_hook_default_project_id,
    mock_base_gcp_hook_no_default_project_id,
)

if TYPE_CHECKING:
    from google.api_core.gapic_v1.method import _MethodDefault

TEST_GCP_CONN_ID: str = "test-gcp-conn-id"
TEST_LOCATION: str = "europe-west-3b"
TEST_ENTRY_ID: str = "test-entry-id"
TEST_ENTRY: dict = {}
TEST_RETRY: Retry | _MethodDefault = Retry()
TEST_TIMEOUT: float = 4
TEST_METADATA: Sequence[tuple[str, str]] = ()
TEST_ENTRY_GROUP_ID: str = "test-entry-group-id"
TEST_ENTRY_GROUP: dict = {}
TEST_TAG: dict = {}
TEST_TAG_TEMPLATE_ID: str = "test-tag-template-id"
TEST_TAG_TEMPLATE: dict = {"name": TEST_TAG_TEMPLATE_ID}
TEST_TAG_TEMPLATE_FIELD_ID: str = "test-tag-template-field-id"
TEST_TAG_TEMPLATE_FIELD: dict = {}
TEST_FORCE: bool = False
TEST_READ_MASK: FieldMask = FieldMask(paths=["name"])
TEST_RESOURCE: str = "test-resource"
TEST_PAGE_SIZE: int = 50
TEST_LINKED_RESOURCE: str = "test-linked-resource"
TEST_SQL_RESOURCE: str = "test-sql-resource"
TEST_NEW_TAG_TEMPLATE_FIELD_ID: str = "test-new-tag-template-field-id"
TEST_SCOPE: dict = {"include_project_ids": ["example-scope-project"]}
TEST_QUERY: str = "test-query"
TEST_ORDER_BY: str = "test-order-by"
TEST_UPDATE_MASK: dict = {"fields": ["name"]}
TEST_PARENT: str = "test-parent"
TEST_NAME: str = "test-name"
TEST_TAG_ID: str = "test-tag-id"
TEST_LOCATION_PATH: str = f"projects/{{}}/locations/{TEST_LOCATION}"
TEST_ENTRY_PATH: str = (
    f"projects/{{}}/locations/{TEST_LOCATION}/entryGroups/{TEST_ENTRY_GROUP_ID}/entries/{TEST_ENTRY_ID}"
)
TEST_ENTRY_GROUP_PATH: str = f"projects/{{}}/locations/{TEST_LOCATION}/entryGroups/{TEST_ENTRY_GROUP_ID}"
TEST_TAG_TEMPLATE_PATH: str = f"projects/{{}}/locations/{TEST_LOCATION}/tagTemplates/{TEST_TAG_TEMPLATE_ID}"
TEST_TAG_TEMPLATE_FIELD_PATH: str = (
    f"projects/{{}}/locations/{TEST_LOCATION}/tagTemplates/"
    f"{TEST_TAG_TEMPLATE_ID}/fields/{TEST_TAG_TEMPLATE_FIELD_ID}"
)
TEST_TAG_PATH: str = (
    f"projects/{{}}/locations/{TEST_LOCATION}/entryGroups/{TEST_ENTRY_GROUP_ID}"
    f"/entries/{TEST_ENTRY_ID}/tags/{TEST_TAG_ID}"
)
TEST_PROJECT_ID_1 = "example-project-1"
TEST_PROJECT_ID_2 = "example-project-2"
TEST_CREDENTIALS = mock.MagicMock()


class TestCloudDataCatalog:
    def setup_method(self):
        with pytest.warns(AirflowProviderDeprecationWarning):
            with mock.patch(
                "airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.__init__",
                new=mock_base_gcp_hook_default_project_id,
            ):
                self.hook = CloudDataCatalogHook(gcp_conn_id="test")

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_lookup_entry_with_linked_resource(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.lookup_entry(
            linked_resource=TEST_LINKED_RESOURCE,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.lookup_entry.assert_called_once_with(
            request=dict(linked_resource=TEST_LINKED_RESOURCE),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_lookup_entry_with_sql_resource(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.lookup_entry(
            sql_resource=TEST_SQL_RESOURCE, retry=TEST_RETRY, timeout=TEST_TIMEOUT, metadata=TEST_METADATA
        )
        mock_get_conn.return_value.lookup_entry.assert_called_once_with(
            request=dict(sql_resource=TEST_SQL_RESOURCE),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_lookup_entry_without_resource(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(
            AirflowException, match=re.escape("At least one of linked_resource, sql_resource should be set.")
        ):
            self.hook.lookup_entry(retry=TEST_RETRY, timeout=TEST_TIMEOUT, metadata=TEST_METADATA)

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_search_catalog(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.search_catalog(
            scope=TEST_SCOPE,
            query=TEST_QUERY,
            page_size=TEST_PAGE_SIZE,
            order_by=TEST_ORDER_BY,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.search_catalog.assert_called_once_with(
            request=dict(
                scope=TEST_SCOPE, query=TEST_QUERY, page_size=TEST_PAGE_SIZE, order_by=TEST_ORDER_BY
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )


class TestCloudDataCatalogWithDefaultProjectIdHook:
    def setup_method(self):
        with pytest.warns(AirflowProviderDeprecationWarning):
            with mock.patch(
                "airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.__init__",
                new=mock_base_gcp_hook_default_project_id,
            ):
                self.hook = CloudDataCatalogHook(gcp_conn_id="test")

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_entry(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry_id=TEST_ENTRY_ID,
            entry=TEST_ENTRY,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_entry.assert_called_once_with(
            request=dict(
                parent=TEST_ENTRY_GROUP_PATH.format(TEST_PROJECT_ID_1),
                entry_id=TEST_ENTRY_ID,
                entry=TEST_ENTRY,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_entry_group(
            location=TEST_LOCATION,
            entry_group_id=TEST_ENTRY_GROUP_ID,
            entry_group=TEST_ENTRY_GROUP,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_entry_group.assert_called_once_with(
            request=dict(
                parent=TEST_LOCATION_PATH.format(TEST_PROJECT_ID_1),
                entry_group_id=TEST_ENTRY_GROUP_ID,
                entry_group=TEST_ENTRY_GROUP,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag=deepcopy(TEST_TAG),
            template_id=TEST_TAG_TEMPLATE_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag.assert_called_once_with(
            request=CreateTagRequest(
                parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1),
                tag=Tag(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1)),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_protobuff(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag=Tag(),
            template_id=TEST_TAG_TEMPLATE_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag.assert_called_once_with(
            request=CreateTagRequest(
                parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1),
                tag=Tag(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1)),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag_template(
            location=TEST_LOCATION,
            tag_template_id=TEST_TAG_TEMPLATE_ID,
            tag_template=TEST_TAG_TEMPLATE,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag_template.assert_called_once_with(
            request=CreateTagTemplateRequest(
                parent=TEST_LOCATION_PATH.format(TEST_PROJECT_ID_1),
                tag_template_id=TEST_TAG_TEMPLATE_ID,
                tag_template=TEST_TAG_TEMPLATE,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag_template_field(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
            tag_template_field=TEST_TAG_TEMPLATE_FIELD,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag_template_field.assert_called_once_with(
            request=dict(
                parent=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1),
                tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
                tag_template_field=TEST_TAG_TEMPLATE_FIELD,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_entry(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_entry.assert_called_once_with(
            request=dict(
                name=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_entry_group(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_entry_group.assert_called_once_with(
            request=dict(
                name=TEST_ENTRY_GROUP_PATH.format(TEST_PROJECT_ID_1),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_tag(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag=TEST_TAG_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_tag.assert_called_once_with(
            request=dict(
                name=TEST_TAG_PATH.format(TEST_PROJECT_ID_1),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_tag_template(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            force=TEST_FORCE,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_tag_template.assert_called_once_with(
            request=dict(name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1), force=TEST_FORCE),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_tag_template_field(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            field=TEST_TAG_TEMPLATE_FIELD_ID,
            force=TEST_FORCE,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_tag_template_field.assert_called_once_with(
            request=dict(
                name=TEST_TAG_TEMPLATE_FIELD_PATH.format(TEST_PROJECT_ID_1),
                force=TEST_FORCE,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.get_entry(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.get_entry.assert_called_once_with(
            request=dict(
                name=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.get_entry_group(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            read_mask=TEST_READ_MASK,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.get_entry_group.assert_called_once_with(
            request=dict(
                name=TEST_ENTRY_GROUP_PATH.format(TEST_PROJECT_ID_1),
                read_mask=TEST_READ_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.get_tag_template(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.get_tag_template.assert_called_once_with(
            request=dict(
                name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_list_tags(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.list_tags(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            page_size=TEST_PAGE_SIZE,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.list_tags.assert_called_once_with(
            request=dict(
                parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1),
                page_size=TEST_PAGE_SIZE,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_tag_for_template_name(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        tag_1 = mock.MagicMock(template=TEST_TAG_TEMPLATE_PATH.format("invalid-project"))
        tag_2 = mock.MagicMock(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1))

        mock_get_conn.return_value.list_tags.return_value = [tag_1, tag_2]
        result = self.hook.get_tag_for_template_name(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            template_name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.list_tags.assert_called_once_with(
            request=dict(
                parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1),
                page_size=100,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        assert result == tag_2

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_rename_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.rename_tag_template_field(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            field=TEST_TAG_TEMPLATE_FIELD_ID,
            new_tag_template_field_id=TEST_NEW_TAG_TEMPLATE_FIELD_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.rename_tag_template_field.assert_called_once_with(
            request=dict(
                name=TEST_TAG_TEMPLATE_FIELD_PATH.format(TEST_PROJECT_ID_1),
                new_tag_template_field_id=TEST_NEW_TAG_TEMPLATE_FIELD_ID,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_entry(
            entry=TEST_ENTRY,
            update_mask=TEST_UPDATE_MASK,
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry_id=TEST_ENTRY_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_entry.assert_called_once_with(
            request=dict(
                entry=Entry(name=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_1)),
                update_mask=TEST_UPDATE_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_tag(
            tag=deepcopy(TEST_TAG),
            update_mask=TEST_UPDATE_MASK,
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag_id=TEST_TAG_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_tag.assert_called_once_with(
            request=dict(tag=Tag(name=TEST_TAG_PATH.format(TEST_PROJECT_ID_1)), update_mask=TEST_UPDATE_MASK),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_tag_template(
            tag_template=TEST_TAG_TEMPLATE,
            update_mask=TEST_UPDATE_MASK,
            location=TEST_LOCATION,
            tag_template_id=TEST_TAG_TEMPLATE_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_tag_template.assert_called_once_with(
            request=dict(
                tag_template=TagTemplate(name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_1)),
                update_mask=TEST_UPDATE_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, TEST_PROJECT_ID_1),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_tag_template_field(
            tag_template_field=TEST_TAG_TEMPLATE_FIELD,
            update_mask=TEST_UPDATE_MASK,
            tag_template=TEST_TAG_TEMPLATE_ID,
            location=TEST_LOCATION,
            tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_tag_template_field.assert_called_once_with(
            request=dict(
                name=TEST_TAG_TEMPLATE_FIELD_PATH.format(TEST_PROJECT_ID_1),
                tag_template_field=TEST_TAG_TEMPLATE_FIELD,
                update_mask=TEST_UPDATE_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )


class TestCloudDataCatalogWithoutDefaultProjectIdHook:
    def setup_method(self):
        with pytest.warns(AirflowProviderDeprecationWarning):
            with mock.patch(
                "airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.__init__",
                new=mock_base_gcp_hook_no_default_project_id,
            ):
                self.hook = CloudDataCatalogHook(gcp_conn_id="test")

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_entry(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry_id=TEST_ENTRY_ID,
            entry=TEST_ENTRY,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_entry.assert_called_once_with(
            request=dict(
                parent=TEST_ENTRY_GROUP_PATH.format(TEST_PROJECT_ID_2),
                entry_id=TEST_ENTRY_ID,
                entry=TEST_ENTRY,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_entry_group(
            location=TEST_LOCATION,
            entry_group_id=TEST_ENTRY_GROUP_ID,
            entry_group=TEST_ENTRY_GROUP,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_entry_group.assert_called_once_with(
            request=dict(
                parent=TEST_LOCATION_PATH.format(TEST_PROJECT_ID_2),
                entry_group_id=TEST_ENTRY_GROUP_ID,
                entry_group=TEST_ENTRY_GROUP,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag=deepcopy(TEST_TAG),
            template_id=TEST_TAG_TEMPLATE_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag.assert_called_once_with(
            request=CreateTagRequest(
                parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2),
                tag=Tag(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2)),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_protobuff(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag=Tag(),
            template_id=TEST_TAG_TEMPLATE_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag.assert_called_once_with(
            request=CreateTagRequest(
                parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2),
                tag=Tag(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2)),
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag_template(
            location=TEST_LOCATION,
            tag_template_id=TEST_TAG_TEMPLATE_ID,
            tag_template=TEST_TAG_TEMPLATE,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag_template.assert_called_once_with(
            request=CreateTagTemplateRequest(
                parent=TEST_LOCATION_PATH.format(TEST_PROJECT_ID_2),
                tag_template_id=TEST_TAG_TEMPLATE_ID,
                tag_template=TEST_TAG_TEMPLATE,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.create_tag_template_field(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
            tag_template_field=TEST_TAG_TEMPLATE_FIELD,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.create_tag_template_field.assert_called_once_with(
            request=dict(
                parent=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2),
                tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
                tag_template_field=TEST_TAG_TEMPLATE_FIELD,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_entry(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_entry.assert_called_once_with(
            request=dict(name=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2)),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_entry_group(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_entry_group.assert_called_once_with(
            request=dict(name=TEST_ENTRY_GROUP_PATH.format(TEST_PROJECT_ID_2)),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_tag(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag=TEST_TAG_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_tag.assert_called_once_with(
            request=dict(name=TEST_TAG_PATH.format(TEST_PROJECT_ID_2)),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_tag_template(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            force=TEST_FORCE,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_tag_template.assert_called_once_with(
            request=dict(name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2), force=TEST_FORCE),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.delete_tag_template_field(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            field=TEST_TAG_TEMPLATE_FIELD_ID,
            force=TEST_FORCE,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.delete_tag_template_field.assert_called_once_with(
            request=dict(name=TEST_TAG_TEMPLATE_FIELD_PATH.format(TEST_PROJECT_ID_2), force=TEST_FORCE),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.get_entry(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.get_entry.assert_called_once_with(
            request=dict(name=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2)),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.get_entry_group(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            read_mask=TEST_READ_MASK,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.get_entry_group.assert_called_once_with(
            request=dict(
                name=TEST_ENTRY_GROUP_PATH.format(TEST_PROJECT_ID_2),
                read_mask=TEST_READ_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.get_tag_template(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.get_tag_template.assert_called_once_with(
            request=dict(name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2)),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_list_tags(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.list_tags(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            page_size=TEST_PAGE_SIZE,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.list_tags.assert_called_once_with(
            request=dict(parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2), page_size=TEST_PAGE_SIZE),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_tag_for_template_name(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        tag_1 = mock.MagicMock(template=TEST_TAG_TEMPLATE_PATH.format("invalid-project"))
        tag_2 = mock.MagicMock(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2))

        mock_get_conn.return_value.list_tags.return_value = [tag_1, tag_2]
        result = self.hook.get_tag_for_template_name(
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            template_name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2),
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.list_tags.assert_called_once_with(
            request=dict(parent=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2), page_size=100),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        assert result == tag_2

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_rename_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.rename_tag_template_field(
            location=TEST_LOCATION,
            tag_template=TEST_TAG_TEMPLATE_ID,
            field=TEST_TAG_TEMPLATE_FIELD_ID,
            new_tag_template_field_id=TEST_NEW_TAG_TEMPLATE_FIELD_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.rename_tag_template_field.assert_called_once_with(
            request=dict(
                name=TEST_TAG_TEMPLATE_FIELD_PATH.format(TEST_PROJECT_ID_2),
                new_tag_template_field_id=TEST_NEW_TAG_TEMPLATE_FIELD_ID,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_entry(
            entry=TEST_ENTRY,
            update_mask=TEST_UPDATE_MASK,
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry_id=TEST_ENTRY_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_entry.assert_called_once_with(
            request=dict(
                entry=Entry(name=TEST_ENTRY_PATH.format(TEST_PROJECT_ID_2)), update_mask=TEST_UPDATE_MASK
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_tag(
            tag=deepcopy(TEST_TAG),
            update_mask=TEST_UPDATE_MASK,
            location=TEST_LOCATION,
            entry_group=TEST_ENTRY_GROUP_ID,
            entry=TEST_ENTRY_ID,
            tag_id=TEST_TAG_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_tag.assert_called_once_with(
            request=dict(tag=Tag(name=TEST_TAG_PATH.format(TEST_PROJECT_ID_2)), update_mask=TEST_UPDATE_MASK),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_tag_template(
            tag_template=TEST_TAG_TEMPLATE,
            update_mask=TEST_UPDATE_MASK,
            location=TEST_LOCATION,
            tag_template_id=TEST_TAG_TEMPLATE_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_tag_template.assert_called_once_with(
            request=dict(
                tag_template=TagTemplate(name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2)),
                update_mask=TEST_UPDATE_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        self.hook.update_tag_template_field(
            tag_template_field=TEST_TAG_TEMPLATE_FIELD,
            update_mask=TEST_UPDATE_MASK,
            tag_template=TEST_TAG_TEMPLATE_ID,
            location=TEST_LOCATION,
            tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
            project_id=TEST_PROJECT_ID_2,
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )
        mock_get_conn.return_value.update_tag_template_field.assert_called_once_with(
            request=dict(
                name=TEST_TAG_TEMPLATE_FIELD_PATH.format(TEST_PROJECT_ID_2),
                tag_template_field=TEST_TAG_TEMPLATE_FIELD,
                update_mask=TEST_UPDATE_MASK,
            ),
            retry=TEST_RETRY,
            timeout=TEST_TIMEOUT,
            metadata=TEST_METADATA,
        )


TEST_MESSAGE = re.escape(
    "The project id must be passed either as keyword project_id parameter or as project_id extra in "
    "Google Cloud connection definition. Both are not set!"
)


class TestCloudDataCatalogMissingProjectIdHook:
    def setup_method(self):
        with pytest.warns(AirflowProviderDeprecationWarning):
            with mock.patch(
                "airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.__init__",
                new=mock_base_gcp_hook_no_default_project_id,
            ):
                self.hook = CloudDataCatalogHook(gcp_conn_id="test")

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.create_entry(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry_id=TEST_ENTRY_ID,
                entry=TEST_ENTRY,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.create_entry_group(
                location=TEST_LOCATION,
                entry_group_id=TEST_ENTRY_GROUP_ID,
                entry_group=TEST_ENTRY_GROUP,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.create_tag(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                tag=deepcopy(TEST_TAG),
                template_id=TEST_TAG_TEMPLATE_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_protobuff(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.create_tag(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                tag=Tag(),
                template_id=TEST_TAG_TEMPLATE_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.create_tag_template(
                location=TEST_LOCATION,
                tag_template_id=TEST_TAG_TEMPLATE_ID,
                tag_template=TEST_TAG_TEMPLATE,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_create_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.create_tag_template_field(
                location=TEST_LOCATION,
                tag_template=TEST_TAG_TEMPLATE_ID,
                tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
                tag_template_field=TEST_TAG_TEMPLATE_FIELD,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.delete_entry(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.delete_entry_group(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.delete_tag(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                tag=TEST_TAG_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.delete_tag_template(
                location=TEST_LOCATION,
                tag_template=TEST_TAG_TEMPLATE_ID,
                force=TEST_FORCE,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_delete_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.delete_tag_template_field(
                location=TEST_LOCATION,
                tag_template=TEST_TAG_TEMPLATE_ID,
                field=TEST_TAG_TEMPLATE_FIELD_ID,
                force=TEST_FORCE,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.get_entry(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_entry_group(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.get_entry_group(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                read_mask=TEST_READ_MASK,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.get_tag_template(
                location=TEST_LOCATION,
                tag_template=TEST_TAG_TEMPLATE_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_list_tags(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.list_tags(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                page_size=TEST_PAGE_SIZE,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_get_tag_for_template_name(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        tag_1 = mock.MagicMock(template=TEST_TAG_TEMPLATE_PATH.format("invalid-project"))
        tag_2 = mock.MagicMock(template=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2))

        mock_get_conn.return_value.list_tags.return_value = [tag_1, tag_2]
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.get_tag_for_template_name(
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                template_name=TEST_TAG_TEMPLATE_PATH.format(TEST_PROJECT_ID_2),
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_rename_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.rename_tag_template_field(
                location=TEST_LOCATION,
                tag_template=TEST_TAG_TEMPLATE_ID,
                field=TEST_TAG_TEMPLATE_FIELD_ID,
                new_tag_template_field_id=TEST_NEW_TAG_TEMPLATE_FIELD_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_entry(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.update_entry(
                entry=TEST_ENTRY,
                update_mask=TEST_UPDATE_MASK,
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry_id=TEST_ENTRY_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.update_tag(
                tag=deepcopy(TEST_TAG),
                update_mask=TEST_UPDATE_MASK,
                location=TEST_LOCATION,
                entry_group=TEST_ENTRY_GROUP_ID,
                entry=TEST_ENTRY_ID,
                tag_id=TEST_TAG_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag_template(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.update_tag_template(
                tag_template=TEST_TAG_TEMPLATE,
                update_mask=TEST_UPDATE_MASK,
                location=TEST_LOCATION,
                tag_template_id=TEST_TAG_TEMPLATE_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials_and_project_id",
        return_value=(TEST_CREDENTIALS, None),
    )
    @mock.patch("airflow.providers.google.cloud.hooks.datacatalog.CloudDataCatalogHook.get_conn")
    def test_update_tag_template_field(self, mock_get_conn, mock_get_creds_and_project_id) -> None:
        with pytest.raises(AirflowException, match=TEST_MESSAGE):
            self.hook.update_tag_template_field(
                tag_template_field=TEST_TAG_TEMPLATE_FIELD,
                update_mask=TEST_UPDATE_MASK,
                tag_template=TEST_TAG_TEMPLATE_ID,
                location=TEST_LOCATION,
                tag_template_field_id=TEST_TAG_TEMPLATE_FIELD_ID,
                retry=TEST_RETRY,
                timeout=TEST_TIMEOUT,
                metadata=TEST_METADATA,
            )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import TYPE_CHECKING, Any

from airflow.providers.google.leveldb.hooks.leveldb import LevelDBHook
from airflow.providers.google.version_compat import BaseOperator

if TYPE_CHECKING:
    from airflow.providers.common.compat.sdk import Context


class LevelDBOperator(BaseOperator):
    """
    Execute command in LevelDB.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:LevelDBOperator`

        :param command: command of plyvel(python wrap for leveldb) for DB object e.g.
            ``"put"``, ``"get"``, ``"delete"``, ``"write_batch"``.
        :param key: key for command(put,get,delete) execution(, e.g. ``b'key'``, ``b'another-key'``)
        :param value: value for command(put) execution(bytes, e.g. ``b'value'``, ``b'another-value'``)
        :param keys: keys for command(write_batch) execution(list[bytes], e.g. ``[b'key', b'another-key'])``
        :param values: values for command(write_batch) execution e.g. ``[b'value'``, ``b'another-value']``
        :param leveldb_conn_id:
        :param create_if_missing: whether a new database should be created if needed
        :param create_db_extra_options: extra options of creation LevelDBOperator. See more in the link below
            `Plyvel DB <https://plyvel.readthedocs.io/en/latest/api.html#DB>`__
    """

    def __init__(
        self,
        *,
        command: str,
        key: bytes,
        value: bytes | None = None,
        keys: list[bytes] | None = None,
        values: list[bytes] | None = None,
        leveldb_conn_id: str = "leveldb_default",
        name: str = "/tmp/testdb/",
        create_if_missing: bool = True,
        create_db_extra_options: dict[str, Any] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.command = command
        self.key = key
        self.value = value
        self.keys = keys
        self.values = values
        self.leveldb_conn_id = leveldb_conn_id
        self.name = name
        self.create_if_missing = create_if_missing
        self.create_db_extra_options = create_db_extra_options or {}

    def execute(self, context: Context) -> str | None:
        """
        Execute command in LevelDB.

        :returns: value from get(str, not bytes, to prevent error in json.dumps in serialize_value in xcom.py)
            or str | None
        """
        leveldb_hook = LevelDBHook(leveldb_conn_id=self.leveldb_conn_id)
        leveldb_hook.get_conn(
            name=self.name, create_if_missing=self.create_if_missing, **self.create_db_extra_options
        )
        value = leveldb_hook.run(
            command=self.command,
            key=self.key,
            value=self.value,
            keys=self.keys,
            values=self.values,
        )
        self.log.info("Done. Returned value was: %s", value)
        leveldb_hook.close_conn()
        str_value = value if value is None else value.decode()
        return str_value



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

import asyncio
import logging
from unittest import mock

import pytest
from google.cloud.dataflow_v1beta3 import Job, JobState, JobType

from airflow.providers.google.cloud.hooks.dataflow import DataflowJobStatus
from airflow.providers.google.cloud.triggers.dataflow import (
    DataflowJobAutoScalingEventTrigger,
    DataflowJobMessagesTrigger,
    DataflowJobMetricsTrigger,
    DataflowJobStateCompleteTrigger,
    DataflowJobStatusTrigger,
    DataflowStartYamlJobTrigger,
    TemplateJobStartTrigger,
)
from airflow.triggers.base import TriggerEvent

PROJECT_ID = "test-project-id"
JOB_ID = "test_job_id_2012-12-23-10:00"
LOCATION = "us-central1"
GCP_CONN_ID = "test_gcp_conn_id"
POLL_SLEEP = 20
IMPERSONATION_CHAIN = ["impersonate", "this"]
CANCEL_TIMEOUT = 10 * 420
WAIT_UNTIL_FINISHED = None


@pytest.fixture
def template_job_start_trigger():
    return TemplateJobStartTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
        cancel_timeout=CANCEL_TIMEOUT,
    )


@pytest.fixture
def dataflow_job_autoscaling_event_trigger():
    return DataflowJobAutoScalingEventTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
        fail_on_terminal_state=False,
    )


@pytest.fixture
def dataflow_job_messages_trigger():
    return DataflowJobMessagesTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
        fail_on_terminal_state=False,
    )


@pytest.fixture
def dataflow_job_metrics_trigger():
    return DataflowJobMetricsTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
        fail_on_terminal_state=False,
    )


@pytest.fixture
def dataflow_job_status_trigger():
    return DataflowJobStatusTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        expected_statuses={JobState.JOB_STATE_DONE, JobState.JOB_STATE_FAILED},
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
    )


@pytest.fixture
def dataflow_start_yaml_job_trigger():
    return DataflowStartYamlJobTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
        cancel_timeout=CANCEL_TIMEOUT,
    )


@pytest.fixture
def dataflow_job_state_complete_trigger():
    return DataflowJobStateCompleteTrigger(
        project_id=PROJECT_ID,
        job_id=JOB_ID,
        location=LOCATION,
        gcp_conn_id=GCP_CONN_ID,
        poll_sleep=POLL_SLEEP,
        impersonation_chain=IMPERSONATION_CHAIN,
        wait_until_finished=WAIT_UNTIL_FINISHED,
    )


@pytest.fixture
def test_dataflow_batch_job():
    return Job(id=JOB_ID, current_state=JobState.JOB_STATE_DONE, type_=JobType.JOB_TYPE_BATCH)


class TestTemplateJobStartTrigger:
    def test_serialize(self, template_job_start_trigger):
        actual_data = template_job_start_trigger.serialize()
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.TemplateJobStartTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "location": LOCATION,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "impersonation_chain": IMPERSONATION_CHAIN,
                "cancel_timeout": CANCEL_TIMEOUT,
            },
        )
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
            ("cancel_timeout", CANCEL_TIMEOUT),
        ],
    )
    def test_get_async_hook(self, template_job_start_trigger, attr, expected):
        hook = template_job_start_trigger._get_async_hook()
        actual = hook._hook_kwargs.get(attr)
        assert actual is not None
        assert actual == expected

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_loop_return_success_event(self, mock_job_status, template_job_start_trigger):
        mock_job_status.return_value = JobState.JOB_STATE_DONE

        expected_event = TriggerEvent(
            {
                "job_id": JOB_ID,
                "status": "success",
                "message": "Job completed",
            }
        )
        actual_event = await template_job_start_trigger.run().asend(None)

        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_loop_return_failed_event(self, mock_job_status, template_job_start_trigger):
        mock_job_status.return_value = JobState.JOB_STATE_FAILED

        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": f"Dataflow job with id {JOB_ID} has failed its execution",
            }
        )
        actual_event = await template_job_start_trigger.run().asend(None)

        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_loop_return_stopped_event(self, mock_job_status, template_job_start_trigger):
        mock_job_status.return_value = JobState.JOB_STATE_STOPPED
        expected_event = TriggerEvent(
            {
                "status": "stopped",
                "message": f"Dataflow job with id {JOB_ID} was stopped",
            }
        )
        actual_event = await template_job_start_trigger.run().asend(None)

        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_loop_is_still_running(self, mock_job_status, template_job_start_trigger, caplog):
        mock_job_status.return_value = JobState.JOB_STATE_RUNNING
        caplog.set_level(logging.INFO)

        task = asyncio.create_task(template_job_start_trigger.run().__anext__())
        await asyncio.sleep(0.5)

        assert not task.done()
        assert "Current job status is: JOB_STATE_RUNNING" in caplog.text
        assert f"Sleeping for {POLL_SLEEP} seconds." in caplog.text
        # cancel the task to suppress test warnings
        task.cancel()


class TestDataflowJobAutoScalingEventTrigger:
    def test_serialize(self, dataflow_job_autoscaling_event_trigger):
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.DataflowJobAutoScalingEventTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "location": LOCATION,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "impersonation_chain": IMPERSONATION_CHAIN,
                "fail_on_terminal_state": False,
            },
        )
        actual_data = dataflow_job_autoscaling_event_trigger.serialize()
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
        ],
    )
    def test_async_hook(self, dataflow_job_autoscaling_event_trigger, attr, expected):
        hook = dataflow_job_autoscaling_event_trigger.async_hook
        actual = hook._hook_kwargs.get(attr)
        assert actual == expected

    @pytest.mark.parametrize(
        "job_status_value",
        [
            JobState.JOB_STATE_DONE,
            JobState.JOB_STATE_FAILED,
            JobState.JOB_STATE_CANCELLED,
            JobState.JOB_STATE_UPDATED,
            JobState.JOB_STATE_DRAINED,
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch(
        "airflow.providers.google.cloud.triggers.dataflow.DataflowJobAutoScalingEventTrigger.list_job_autoscaling_events"
    )
    async def test_run_yields_terminal_state_event_if_fail_on_terminal_state(
        self,
        mock_list_job_autoscaling_events,
        mock_job_status,
        job_status_value,
        dataflow_job_autoscaling_event_trigger,
    ):
        dataflow_job_autoscaling_event_trigger.fail_on_terminal_state = True
        mock_list_job_autoscaling_events.return_value = []
        mock_job_status.return_value = job_status_value
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": f"Job with id '{JOB_ID}' is already in terminal state: {job_status_value.name}",
                "result": None,
            }
        )
        actual_event = await dataflow_job_autoscaling_event_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch(
        "airflow.providers.google.cloud.triggers.dataflow.DataflowJobAutoScalingEventTrigger.list_job_autoscaling_events"
    )
    async def test_run_loop_is_still_running_if_fail_on_terminal_state(
        self,
        mock_list_job_autoscaling_events,
        mock_job_status,
        dataflow_job_autoscaling_event_trigger,
        caplog,
    ):
        """Test that DataflowJobAutoScalingEventTrigger is still in loop if the job status is RUNNING."""
        dataflow_job_autoscaling_event_trigger.fail_on_terminal_state = True
        mock_job_status.return_value = JobState.JOB_STATE_RUNNING
        mock_list_job_autoscaling_events.return_value = []
        caplog.set_level(logging.INFO)
        task = asyncio.create_task(dataflow_job_autoscaling_event_trigger.run().__anext__())
        await asyncio.sleep(0.5)
        assert task.done() is False
        # cancel the task to suppress test warnings
        task.cancel()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch(
        "airflow.providers.google.cloud.triggers.dataflow.DataflowJobAutoScalingEventTrigger.list_job_autoscaling_events"
    )
    async def test_run_yields_autoscaling_events(
        self, mock_list_job_autoscaling_events, mock_job_status, dataflow_job_autoscaling_event_trigger
    ):
        mock_job_status.return_value = JobState.JOB_STATE_DONE
        test_autoscaling_events = [
            {
                "event_type": 2,
                "description": {},
                "time": "2024-02-05T13:43:31.066611771Z",
                "worker_pool": "Regular",
                "current_num_workers": "0",
                "target_num_workers": "0",
            },
            {
                "target_num_workers": "1",
                "event_type": 1,
                "description": {},
                "time": "2024-02-05T13:43:31.066611771Z",
                "worker_pool": "Regular",
                "current_num_workers": "0",
            },
        ]
        mock_list_job_autoscaling_events.return_value = test_autoscaling_events
        expected_event = TriggerEvent(
            {
                "status": "success",
                "message": f"Detected 2 autoscaling events for job '{JOB_ID}'",
                "result": test_autoscaling_events,
            }
        )
        actual_event = await dataflow_job_autoscaling_event_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_raises_exception(self, mock_job_status, dataflow_job_autoscaling_event_trigger):
        """
        Tests the DataflowJobAutoScalingEventTrigger does fire if there is an exception.
        """
        mock_job_status.side_effect = mock.AsyncMock(side_effect=Exception("Test exception"))
        expected_event = TriggerEvent({"status": "error", "message": "Test exception", "result": None})
        actual_event = await dataflow_job_autoscaling_event_trigger.run().asend(None)
        assert expected_event == actual_event


class TestDataflowJobMessagesTrigger:
    """Test case for DataflowJobMessagesTrigger"""

    def test_serialize(self, dataflow_job_messages_trigger):
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.DataflowJobMessagesTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "location": LOCATION,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "impersonation_chain": IMPERSONATION_CHAIN,
                "fail_on_terminal_state": False,
            },
        )
        actual_data = dataflow_job_messages_trigger.serialize()
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
        ],
    )
    def test_async_hook(self, dataflow_job_messages_trigger, attr, expected):
        hook = dataflow_job_messages_trigger.async_hook
        actual = hook._hook_kwargs.get(attr)
        assert actual == expected

    @pytest.mark.parametrize(
        "job_status_value",
        [
            JobState.JOB_STATE_DONE,
            JobState.JOB_STATE_FAILED,
            JobState.JOB_STATE_CANCELLED,
            JobState.JOB_STATE_UPDATED,
            JobState.JOB_STATE_DRAINED,
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch(
        "airflow.providers.google.cloud.triggers.dataflow.DataflowJobMessagesTrigger.list_job_messages"
    )
    async def test_run_yields_terminal_state_event_if_fail_on_terminal_state(
        self,
        mock_list_job_messages,
        mock_job_status,
        job_status_value,
        dataflow_job_messages_trigger,
    ):
        dataflow_job_messages_trigger.fail_on_terminal_state = True
        mock_list_job_messages.return_value = []
        mock_job_status.return_value = job_status_value
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": f"Job with id '{JOB_ID}' is already in terminal state: {job_status_value.name}",
                "result": None,
            }
        )
        actual_event = await dataflow_job_messages_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch(
        "airflow.providers.google.cloud.triggers.dataflow.DataflowJobMessagesTrigger.list_job_messages"
    )
    async def test_run_loop_is_still_running_if_fail_on_terminal_state(
        self,
        mock_list_job_messages,
        mock_job_status,
        dataflow_job_messages_trigger,
        caplog,
    ):
        """Test that DataflowJobMessagesTrigger is still in loop if the job status is RUNNING."""
        dataflow_job_messages_trigger.fail_on_terminal_state = True
        mock_job_status.return_value = JobState.JOB_STATE_RUNNING
        mock_list_job_messages.return_value = []
        caplog.set_level(logging.INFO)
        task = asyncio.create_task(dataflow_job_messages_trigger.run().__anext__())
        await asyncio.sleep(0.5)
        assert task.done() is False
        # cancel the task to suppress test warnings
        task.cancel()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch(
        "airflow.providers.google.cloud.triggers.dataflow.DataflowJobMessagesTrigger.list_job_messages"
    )
    async def test_run_yields_job_messages(
        self, mock_list_job_messages, mock_job_status, dataflow_job_messages_trigger
    ):
        mock_job_status.return_value = JobState.JOB_STATE_DONE
        test_job_messages = [
            {
                "id": "1707695235850",
                "time": "2024-02-06T23:47:15.850Z",
                "message_text": "msg.",
                "message_importance": 5,
            },
            {
                "id": "1707695635401",
                "time": "2024-02-06T23:53:55.401Z",
                "message_text": "msg.",
                "message_importance": 5,
            },
        ]
        mock_list_job_messages.return_value = test_job_messages
        expected_event = TriggerEvent(
            {
                "status": "success",
                "message": f"Detected 2 job messages for job '{JOB_ID}'",
                "result": test_job_messages,
            }
        )
        actual_event = await dataflow_job_messages_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_raises_exception(self, mock_job_status, dataflow_job_messages_trigger):
        """
        Tests the DataflowJobMessagesTrigger does fire if there is an exception.
        """
        mock_job_status.side_effect = mock.AsyncMock(side_effect=Exception("Test exception"))
        expected_event = TriggerEvent({"status": "error", "message": "Test exception", "result": None})
        actual_event = await dataflow_job_messages_trigger.run().asend(None)
        assert expected_event == actual_event


class TestDataflowJobMetricsTrigger:
    """Test case for DataflowJobMetricsTrigger"""

    def test_serialize(self, dataflow_job_metrics_trigger):
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.DataflowJobMetricsTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "location": LOCATION,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "impersonation_chain": IMPERSONATION_CHAIN,
                "fail_on_terminal_state": False,
            },
        )
        actual_data = dataflow_job_metrics_trigger.serialize()
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
        ],
    )
    def test_async_hook(self, dataflow_job_metrics_trigger, attr, expected):
        hook = dataflow_job_metrics_trigger.async_hook
        actual = hook._hook_kwargs.get(attr)
        assert actual == expected

    @pytest.mark.parametrize(
        "job_status_value",
        [
            JobState.JOB_STATE_DONE,
            JobState.JOB_STATE_FAILED,
            JobState.JOB_STATE_CANCELLED,
            JobState.JOB_STATE_UPDATED,
            JobState.JOB_STATE_DRAINED,
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch("airflow.providers.google.cloud.triggers.dataflow.DataflowJobMetricsTrigger.get_job_metrics")
    async def test_run_yields_terminal_state_event_if_fail_on_terminal_state(
        self,
        mock_get_job_metrics,
        mock_job_status,
        job_status_value,
        dataflow_job_metrics_trigger,
    ):
        dataflow_job_metrics_trigger.fail_on_terminal_state = True
        mock_get_job_metrics.return_value = []
        mock_job_status.return_value = job_status_value
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": f"Job with id '{JOB_ID}' is already in terminal state: {job_status_value.name}",
                "result": None,
            }
        )
        actual_event = await dataflow_job_metrics_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch("airflow.providers.google.cloud.triggers.dataflow.DataflowJobMetricsTrigger.get_job_metrics")
    async def test_run_loop_is_still_running_if_fail_on_terminal_state(
        self,
        mock_get_job_metrics,
        mock_job_status,
        dataflow_job_metrics_trigger,
    ):
        """Test that DataflowJobMetricsTrigger is still in loop if the job status is RUNNING."""
        dataflow_job_metrics_trigger.fail_on_terminal_state = True
        mock_job_status.return_value = JobState.JOB_STATE_RUNNING
        mock_get_job_metrics.return_value = []
        task = asyncio.create_task(dataflow_job_metrics_trigger.run().__anext__())
        await asyncio.sleep(0.5)
        assert task.done() is False
        # cancel the task to suppress test warnings
        task.cancel()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    @mock.patch("airflow.providers.google.cloud.triggers.dataflow.DataflowJobMetricsTrigger.get_job_metrics")
    async def test_run_yields_job_messages(
        self, mock_get_job_metrics, mock_job_status, dataflow_job_metrics_trigger
    ):
        mock_job_status.return_value = JobState.JOB_STATE_DONE
        test_job_metrics = [
            {
                "name": {"origin": "", "name": "", "context": {}},
                "scalar": 0.0,
                "update_time": "2024-03-20T12:36:05.229Z",
                "kind": "",
                "cumulative": False,
            },
            {
                "name": {"origin": "", "name": "", "context": {}},
                "scalar": 0.0,
                "update_time": "2024-03-20T12:36:05.229Z",
                "kind": "",
                "cumulative": False,
            },
        ]
        mock_get_job_metrics.return_value = test_job_metrics
        expected_event = TriggerEvent(
            {
                "status": "success",
                "message": f"Detected 2 metrics for job '{JOB_ID}'",
                "result": test_job_metrics,
            }
        )
        actual_event = await dataflow_job_metrics_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_raises_exception(self, mock_job_status, dataflow_job_metrics_trigger):
        """
        Tests the DataflowJobMetrcisTrigger does fire if there is an exception.
        """
        mock_job_status.side_effect = mock.AsyncMock(side_effect=Exception("Test exception"))
        expected_event = TriggerEvent({"status": "error", "message": "Test exception", "result": None})
        actual_event = await dataflow_job_metrics_trigger.run().asend(None)
        assert expected_event == actual_event


class TestDataflowJobStatusTrigger:
    """Test case for DataflowJobStatusTrigger"""

    def test_serialize(self, dataflow_job_status_trigger):
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.DataflowJobStatusTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "expected_statuses": {JobState.JOB_STATE_DONE, JobState.JOB_STATE_FAILED},
                "location": LOCATION,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "impersonation_chain": IMPERSONATION_CHAIN,
            },
        )
        actual_data = dataflow_job_status_trigger.serialize()
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
        ],
    )
    def test_async_hook(self, dataflow_job_status_trigger, attr, expected):
        hook = dataflow_job_status_trigger.async_hook
        actual = hook._hook_kwargs.get(attr)
        assert actual == expected

    @pytest.mark.parametrize(
        "job_status_value",
        [
            JobState.JOB_STATE_DONE,
            JobState.JOB_STATE_FAILED,
            JobState.JOB_STATE_CANCELLED,
            JobState.JOB_STATE_UPDATED,
            JobState.JOB_STATE_DRAINED,
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_yields_terminal_state_event(
        self,
        mock_job_status,
        job_status_value,
        dataflow_job_status_trigger,
    ):
        dataflow_job_status_trigger.expected_statuses = {DataflowJobStatus.JOB_STATE_CANCELLING}
        mock_job_status.return_value = job_status_value
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": f"Job with id '{JOB_ID}' is already in terminal state: {job_status_value.name}",
            }
        )
        actual_event = await dataflow_job_status_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.parametrize(
        "job_status_value",
        [
            JobState.JOB_STATE_DONE,
            JobState.JOB_STATE_RUNNING,
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_yields_success_event_if_expected_job_status(
        self,
        mock_job_status,
        job_status_value,
        dataflow_job_status_trigger,
    ):
        dataflow_job_status_trigger.expected_statuses = {
            DataflowJobStatus.JOB_STATE_DONE,
            DataflowJobStatus.JOB_STATE_RUNNING,
        }
        mock_job_status.return_value = job_status_value
        expected_event = TriggerEvent(
            {
                "status": "success",
                "message": f"Job with id '{JOB_ID}' has reached an expected state: {job_status_value.name}",
            }
        )
        actual_event = await dataflow_job_status_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_loop_is_still_running_if_state_is_not_terminal_or_expected(
        self,
        mock_job_status,
        dataflow_job_status_trigger,
    ):
        """Test that DataflowJobStatusTrigger is still in loop if the job status neither terminal nor expected."""
        dataflow_job_status_trigger.expected_statuses = {DataflowJobStatus.JOB_STATE_DONE}
        mock_job_status.return_value = JobState.JOB_STATE_RUNNING
        task = asyncio.create_task(dataflow_job_status_trigger.run().__anext__())
        await asyncio.sleep(0.5)
        assert task.done() is False
        task.cancel()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job_status")
    async def test_run_raises_exception(self, mock_job_status, dataflow_job_status_trigger):
        """
        Tests the DataflowJobStatusTrigger does fire if there is an exception.
        """
        mock_job_status.side_effect = mock.AsyncMock(side_effect=Exception("Test exception"))
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": "Test exception",
            }
        )
        actual_event = await dataflow_job_status_trigger.run().asend(None)
        assert expected_event == actual_event


class TestDataflowStartYamlJobTrigger:
    def test_serialize(self, dataflow_start_yaml_job_trigger):
        actual_data = dataflow_start_yaml_job_trigger.serialize()
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.DataflowStartYamlJobTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "location": LOCATION,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "expected_terminal_state": None,
                "impersonation_chain": IMPERSONATION_CHAIN,
                "cancel_timeout": CANCEL_TIMEOUT,
            },
        )
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
            ("cancel_timeout", CANCEL_TIMEOUT),
        ],
    )
    def test_get_async_hook(self, dataflow_start_yaml_job_trigger, attr, expected):
        hook = dataflow_start_yaml_job_trigger._get_async_hook()
        actual = hook._hook_kwargs.get(attr)
        assert actual is not None
        assert actual == expected

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_loop_return_success_event(
        self, mock_get_job, dataflow_start_yaml_job_trigger, test_dataflow_batch_job
    ):
        mock_get_job.return_value = test_dataflow_batch_job
        expected_event = TriggerEvent(
            {
                "job": Job.to_dict(test_dataflow_batch_job),
                "status": "success",
                "message": "Batch job completed.",
            }
        )
        actual_event = await dataflow_start_yaml_job_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_loop_return_failed_event(
        self, mock_get_job, dataflow_start_yaml_job_trigger, test_dataflow_batch_job
    ):
        test_dataflow_batch_job.current_state = JobState.JOB_STATE_FAILED
        mock_get_job.return_value = test_dataflow_batch_job
        expected_event = TriggerEvent(
            {
                "job": Job.to_dict(test_dataflow_batch_job),
                "status": "error",
                "message": "Job failed.",
            }
        )
        actual_event = await dataflow_start_yaml_job_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_loop_return_stopped_event(
        self, mock_get_job, dataflow_start_yaml_job_trigger, test_dataflow_batch_job
    ):
        test_dataflow_batch_job.current_state = JobState.JOB_STATE_STOPPED
        mock_get_job.return_value = test_dataflow_batch_job
        expected_event = TriggerEvent(
            {
                "job": Job.to_dict(test_dataflow_batch_job),
                "status": "stopped",
                "message": "Job was stopped.",
            }
        )
        actual_event = await dataflow_start_yaml_job_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_loop_return_expected_state_event(
        self, mock_get_job, dataflow_start_yaml_job_trigger, test_dataflow_batch_job
    ):
        dataflow_start_yaml_job_trigger.expected_terminal_state = DataflowJobStatus.JOB_STATE_RUNNING
        test_dataflow_batch_job.current_state = JobState.JOB_STATE_RUNNING
        mock_get_job.return_value = test_dataflow_batch_job
        expected_event = TriggerEvent(
            {
                "job": Job.to_dict(test_dataflow_batch_job),
                "status": "success",
                "message": f"Job reached the expected terminal state: {DataflowJobStatus.JOB_STATE_RUNNING}.",
            }
        )
        actual_event = await dataflow_start_yaml_job_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_loop_is_still_running(
        self, mock_get_job, dataflow_start_yaml_job_trigger, test_dataflow_batch_job
    ):
        """Test that DataflowStartYamlJobTrigger is still in loop if the job status neither terminal nor expected."""
        dataflow_start_yaml_job_trigger.expected_terminal_state = DataflowJobStatus.JOB_STATE_STOPPED
        test_dataflow_batch_job.current_state = JobState.JOB_STATE_RUNNING
        mock_get_job.return_value = test_dataflow_batch_job
        task = asyncio.create_task(dataflow_start_yaml_job_trigger.run().__anext__())
        await asyncio.sleep(0.5)
        assert task.done() is False
        task.cancel()


class TestDataflowJobStateCompleteTrigger:
    """Test case for DataflowJobStatusTrigger"""

    def test_serialize(self, dataflow_job_state_complete_trigger):
        expected_data = (
            "airflow.providers.google.cloud.triggers.dataflow.DataflowJobStateCompleteTrigger",
            {
                "project_id": PROJECT_ID,
                "job_id": JOB_ID,
                "location": LOCATION,
                "wait_until_finished": WAIT_UNTIL_FINISHED,
                "gcp_conn_id": GCP_CONN_ID,
                "poll_sleep": POLL_SLEEP,
                "impersonation_chain": IMPERSONATION_CHAIN,
            },
        )
        actual_data = dataflow_job_state_complete_trigger.serialize()
        assert actual_data == expected_data

    @pytest.mark.parametrize(
        ("attr", "expected"),
        [
            ("gcp_conn_id", GCP_CONN_ID),
            ("poll_sleep", POLL_SLEEP),
            ("impersonation_chain", IMPERSONATION_CHAIN),
        ],
    )
    def test_async_hook(self, dataflow_job_state_complete_trigger, attr, expected):
        hook = dataflow_job_state_complete_trigger.async_hook
        actual = hook._hook_kwargs.get(attr)
        assert actual == expected

    @pytest.mark.parametrize(
        "job_status_value",
        [
            JobState.JOB_STATE_FAILED,
            JobState.JOB_STATE_CANCELLED,
            JobState.JOB_STATE_DRAINED,
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_yields_error_on_failed_state(
        self,
        mock_job,
        job_status_value,
        dataflow_job_state_complete_trigger,
    ):
        mock_job.return_value = Job(
            id=JOB_ID, current_state=job_status_value, type_=JobType.JOB_TYPE_STREAMING
        )
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": f"Job with id '{JOB_ID}' is in failed terminal state: {job_status_value.name}",
            }
        )
        actual_event = await dataflow_job_state_complete_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.parametrize(
        "job_item",
        [
            Job(id=JOB_ID, current_state=JobState.JOB_STATE_DONE, type_=JobType.JOB_TYPE_BATCH),
            Job(id=JOB_ID, current_state=JobState.JOB_STATE_RUNNING, type_=JobType.JOB_TYPE_STREAMING),
        ],
    )
    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_yields_success_event_if_expected_job_status(
        self,
        mock_job,
        job_item,
        dataflow_job_state_complete_trigger,
    ):
        dataflow_job_state_complete_trigger.expected_statuses = {
            DataflowJobStatus.JOB_STATE_DONE,
            DataflowJobStatus.JOB_STATE_RUNNING,
        }
        mock_job.return_value = mock_job.return_value = job_item

        expected_event = TriggerEvent(
            {
                "status": "success",
                "message": f"Job with id '{JOB_ID}' has reached successful final state:"
                f" {job_item.current_state.name}",
            }
        )
        actual_event = await dataflow_job_state_complete_trigger.run().asend(None)
        assert actual_event == expected_event

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_loop_is_still_running_if_state_is_not_terminal_or_expected(
        self,
        mock_job,
        dataflow_job_state_complete_trigger,
    ):
        """
        Test that DataflowJobStateCompleteTrigger is still in loop if the job status neither
        terminal nor expected.
        """
        mock_job.return_value = Job(
            id=JOB_ID,
            current_state=JobState.JOB_STATE_PENDING,
            type_=JobType.JOB_TYPE_STREAMING,
        )
        task = asyncio.create_task(dataflow_job_state_complete_trigger.run().__anext__())
        await asyncio.sleep(0.5)
        assert task.done() is False
        task.cancel()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.dataflow.AsyncDataflowHook.get_job")
    async def test_run_raises_exception(self, mock_job, dataflow_job_state_complete_trigger):
        """
        Tests the DataflowJobStateCompleteTrigger does trigger error if there is an exception.
        """
        mock_job.side_effect = mock.AsyncMock(side_effect=Exception("Test exception"))
        expected_event = TriggerEvent(
            {
                "status": "error",
                "message": "Test exception",
            }
        )
        actual_event = await dataflow_job_state_complete_trigger.run().asend(None)
        assert expected_event == actual_event



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.exceptions import AirflowException
from airflow.providers.google.suite.transfers.gcs_to_gdrive import GCSToGoogleDriveOperator

MODULE = "airflow.providers.google.suite.transfers.gcs_to_gdrive"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]


class TestGcsToGDriveOperator:
    @mock.patch(MODULE + ".GCSHook")
    @mock.patch(MODULE + ".GoogleDriveHook")
    @mock.patch(MODULE + ".tempfile.NamedTemporaryFile")
    def test_should_copy_single_file(self, mock_named_temporary_file, mock_gdrive, mock_gcs_hook):
        type(mock_named_temporary_file.return_value.__enter__.return_value).name = mock.PropertyMock(
            side_effect=["TMP1"]
        )
        task = GCSToGoogleDriveOperator(
            task_id="copy_single_file",
            source_bucket="data",
            source_object="sales/sales-2017/january.avro",
            destination_object="copied_sales/2017/january-backup.avro",
            impersonation_chain=None,
        )

        task.execute(mock.MagicMock())

        mock_gcs_hook.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=None,
                ),
                mock.call().download(
                    bucket_name="data", filename="TMP1", object_name="sales/sales-2017/january.avro"
                ),
            ]
        )

        mock_gdrive.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=None,
                ),
                mock.call().upload_file(
                    local_location="TMP1",
                    remote_location="copied_sales/2017/january-backup.avro",
                    folder_id="root",
                ),
            ]
        )

    @mock.patch(MODULE + ".GCSHook")
    @mock.patch(MODULE + ".GoogleDriveHook")
    @mock.patch(MODULE + ".tempfile.NamedTemporaryFile")
    def test_should_copy_single_file_with_folder(self, mock_named_temporary_file, mock_gdrive, mock_gcs_hook):
        type(mock_named_temporary_file.return_value.__enter__.return_value).name = mock.PropertyMock(
            side_effect=["TMP1"]
        )
        task = GCSToGoogleDriveOperator(
            task_id="copy_single_file",
            source_bucket="data",
            source_object="sales/sales-2017/january.avro",
            destination_object="copied_sales/2017/january-backup.avro",
            destination_folder_id="aAopls6bE4tUllZVGJvRUU",
        )

        task.execute(mock.MagicMock())

        mock_gcs_hook.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=None,
                ),
                mock.call().download(
                    bucket_name="data", filename="TMP1", object_name="sales/sales-2017/january.avro"
                ),
            ]
        )

        mock_gdrive.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=None,
                ),
                mock.call().upload_file(
                    local_location="TMP1",
                    remote_location="copied_sales/2017/january-backup.avro",
                    folder_id="aAopls6bE4tUllZVGJvRUU",
                ),
            ]
        )

    #
    @mock.patch(MODULE + ".GCSHook")
    @mock.patch(MODULE + ".GoogleDriveHook")
    @mock.patch(MODULE + ".tempfile.NamedTemporaryFile")
    def test_should_copy_files(self, mock_named_temporary_file, mock_gdrive, mock_gcs_hook):
        mock_gcs_hook.return_value.list.return_value = ["sales/A.avro", "sales/B.avro", "sales/C.avro"]
        type(mock_named_temporary_file.return_value.__enter__.return_value).name = mock.PropertyMock(
            side_effect=["TMP1", "TMP2", "TMP3"]
        )

        task = GCSToGoogleDriveOperator(
            task_id="copy_files",
            source_bucket="data",
            source_object="sales/sales-2017/*.avro",
            destination_object="copied_sales/2017/",
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        task.execute(mock.MagicMock())
        mock_gcs_hook.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=IMPERSONATION_CHAIN,
                ),
                mock.call().list("data", delimiter=".avro", prefix="sales/sales-2017/"),
                # TODO: After deprecating delimiter and wildcards in source objects,
                #       remove previous line and uncomment the following:
                # mock.call().list("data", match_glob="**/*.avro", prefix="sales/sales-2017/"),
                mock.call().download(bucket_name="data", filename="TMP1", object_name="sales/A.avro"),
                mock.call().download(bucket_name="data", filename="TMP2", object_name="sales/B.avro"),
                mock.call().download(bucket_name="data", filename="TMP3", object_name="sales/C.avro"),
            ]
        )

        mock_gdrive.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=IMPERSONATION_CHAIN,
                ),
                mock.call().upload_file(
                    local_location="TMP1", remote_location="sales/A.avro", folder_id="root"
                ),
                mock.call().upload_file(
                    local_location="TMP2", remote_location="sales/B.avro", folder_id="root"
                ),
                mock.call().upload_file(
                    local_location="TMP3", remote_location="sales/C.avro", folder_id="root"
                ),
            ]
        )

    @mock.patch(MODULE + ".GCSHook")
    @mock.patch(MODULE + ".GoogleDriveHook")
    @mock.patch(MODULE + ".tempfile.NamedTemporaryFile")
    def test_should_move_files(self, mock_named_temporary_file, mock_gdrive, mock_gcs_hook):
        type(mock_named_temporary_file.return_value.__enter__.return_value).name = mock.PropertyMock(
            side_effect=["TMP1", "TMP2", "TMP3"]
        )
        mock_gcs_hook.return_value.list.return_value = ["sales/A.avro", "sales/B.avro", "sales/C.avro"]
        task = GCSToGoogleDriveOperator(
            task_id="move_files",
            source_bucket="data",
            source_object="sales/sales-2017/*.avro",
            move_object=True,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        task.execute(mock.MagicMock())
        mock_gcs_hook.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=IMPERSONATION_CHAIN,
                ),
                mock.call().list("data", delimiter=".avro", prefix="sales/sales-2017/"),
                # TODO: After deprecating delimiter and wildcards in source objects,
                #       remove previous line and uncomment the following:
                # mock.call().list("data", match_glob="**/*.avro", prefix="sales/sales-2017/"),
                mock.call().download(bucket_name="data", filename="TMP1", object_name="sales/A.avro"),
                mock.call().delete("data", "sales/A.avro"),
                mock.call().download(bucket_name="data", filename="TMP2", object_name="sales/B.avro"),
                mock.call().delete("data", "sales/B.avro"),
                mock.call().download(bucket_name="data", filename="TMP3", object_name="sales/C.avro"),
                mock.call().delete("data", "sales/C.avro"),
            ]
        )

        mock_gdrive.assert_has_calls(
            [
                mock.call(
                    gcp_conn_id="google_cloud_default",
                    impersonation_chain=IMPERSONATION_CHAIN,
                ),
                mock.call().upload_file(
                    local_location="TMP1", remote_location="sales/A.avro", folder_id="root"
                ),
                mock.call().upload_file(
                    local_location="TMP2", remote_location="sales/B.avro", folder_id="root"
                ),
                mock.call().upload_file(
                    local_location="TMP3", remote_location="sales/C.avro", folder_id="root"
                ),
            ]
        )

    @mock.patch(MODULE + ".GCSHook")
    @mock.patch(MODULE + ".GoogleDriveHook")
    @mock.patch(MODULE + ".tempfile.NamedTemporaryFile")
    def test_should_raise_exception_on_multiple_wildcard(
        self, mock_named_temporary_file, mock_gdrive, mock_gcs_hook
    ):
        task = GCSToGoogleDriveOperator(
            task_id="move_files", source_bucket="data", source_object="sales/*/*.avro", move_object=True
        )
        with pytest.raises(AirflowException, match="Only one wildcard"):
            task.execute(mock.MagicMock())
